/**  Reference Manual Help  **/

Components.utils.import('resource://gre/modules/XPCOMUtils.jsm');
Components.utils.import('resource://gre/modules/Services.jsm');
Components.utils.import('resource://gre/modules/FileUtils.jsm'); 
Components.utils.import('resource://gre/modules/AddonManager.jsm');

XPCOMUtils.defineLazyServiceGetter(this, 'JSB', '@kmeleon/jsbridge;1', 'nsIJSBridge');
XPCOMUtils.defineLazyServiceGetter(this, 'as', '@mozilla.org/alerts-service;1', 'nsIAlertsService');
XPCOMUtils.defineLazyServiceGetter(this, 'cs', '@mozilla.org/consoleservice;1', 'nsIConsoleService');
XPCOMUtils.defineLazyGetter(this, 'sstring', function () { 
	return Components.classes['@mozilla.org/supports-string;1']
		.createInstance(Components.interfaces.nsISupportsString);
});
XPCOMUtils.defineLazyGetter(this, 'lstring', function () { 
	return Components.classes['@mozilla.org/pref-localizedstring;1']
		.createInstance(Components.interfaces.nsIPrefLocalizedString);
});

const util = {
	_installed: false,
	get installed () { // unload
		return this._installed;
	},
	set installed (value) {
		this._installed = value;
	},
	_topic: 'kmeleon-init',
	get topic () { // init
		return this._topic;
	},
	jsbload: function (reason) { // init

	/**	startup reason APP_STARTUP, ADDON_ENABLE, ADDON_INSTALL, ADDON_UPGRADE or ADDON_DOWNGRADE
	*
	*	APP_STARTUP     : 1	
	*	APP_SHUTDOWN    : 2
	*	ADDON_ENABLE    : 3
	*	ADDON_DISABLE   : 4
	*	ADDON_INSTALL   : 5
	*	ADDON_UNINSTALL : 6
	*	ADDON_UPGRADE   : 7
	*	ADDON_DOWNGRADE : 8
	*
	**/
		if (reason == 1)
			return (FileUtils.getFile('GreD', ['kplugins', 'JSBridge.dll']).exists() && this.getPref('kmeleon.plugins.jsbridge.load'));
		else
			return (FileUtils.getFile('GreD', ['kplugins', 'JSBridge.dll']).exists() && this.getPref('kmeleon.plugins.jsbridge.load') && ('@kmeleon/jsbridge;1' in Components.classes));
	},
	_addonPath: null,
	get addonPath () { // preferences
		return this._addonPath;
	},
	set addonPath (value) {
		this._addonPath = value;
	},
	getPref: function (name, type) { // preferences
		var ptype = Services.prefs.getPrefType(name);
		try {
			if (type && ptype == 32)
				return Services.prefs.getComplexValue(name, Components.interfaces.nsIPrefLocalizedString).data;
			else if (ptype == 32)
				return Services.prefs.getComplexValue(name, Components.interfaces.nsISupportsString).data;
			else if (ptype == 64)
				return Services.prefs.getIntPref(name);
			else if (ptype == 128)
				return Services.prefs.getBoolPref(name);
		} catch (e) {}
		return null;
	},
	setPref: function (name, value, type) { // preferences
		var ptype = Services.prefs.getPrefType(name);
		try {
			if (type && ptype == 32) {
				lstring.data = value;
				return Services.prefs.setComplexValue(name, Components.interfaces.nsIPrefLocalizedString, lstring);
			} else if (ptype == 32) {
				sstring.data = value;
				Services.prefs.setComplexValue(name, Components.interfaces.nsISupportsString, sstring);
			} else if (ptype == 64)
				Services.prefs.setIntPref(name, value);
			else if (ptype == 128)
				Services.prefs.setBoolPref(name, value);
		} catch (e) {}
	},
	loadDefaultPrefs: function (apath, file) { // preferences
		var pref = function (name, value) {
			try {
				switch (typeof value) {
				case 'boolean':
					Services.prefs.getDefaultBranch('').setBoolPref(name, value);
					break;
				case 'number':
					Services.prefs.getDefaultBranch('').setIntPref(name, value);
					break;
				case 'string':
					sstring.data = value;
					Services.prefs.getDefaultBranch('').setComplexValue(name, Components.interfaces.nsISupportsString, sstring);
					break;
				}
			} catch (e) {
				Components.utils.reportError(util.getString('reportError', NAME) + '\n' + name);
			}
		};
		try {
			var uri, baseURI = Services.io.newFileURI(apath);
			if (apath.isDirectory())
				uri = Services.io.newURI('defaults/preferences/' + file, null, baseURI).spec;
			else
				uri = 'jar:' + baseURI.spec + '!/defaults/preferences/' + file;
			Services.scriptloader.loadSubScript(uri, {pref: pref});
		} catch (e) {
			Components.utils.reportError(e);
		}
	},
	openWindow: function (url, name, features, args, where) {

		/** JSBridge constants
		*
		*	OPEN_NORMAL = 0
		*	OPEN_NEW = 1
		*	OPEN_BACKGROUND = 2
		*	OPEN_NEWTAB = 3
		*	OPEN_BACKGROUNDTAB = 4
		*	OPEN_CLONE = 16
		*	
		*	xul window = 6
		**/

		if (where == 6) {
			var wndlist = Services.ww.getWindowEnumerator();
			while (wndlist.hasMoreElements()) {
				var wnd = wndlist.getNext().QueryInterface(Components.interfaces.nsIDOMWindow);
				if (wnd.name == name)
					return wnd.focus();
			}
			try {
				Services.ww.openWindow(JSB.GetActiveBrowser().contentDOMWindow, url, name, features, args);
			} catch (e) {}
		} else
			JSB.Open(url, where);
	},
	getString: function (string, rstring) { // properties alert, init
		var vstring;
		if (rstring)
			vstring = rstring;
		else
			vstring = this.Brand.GetStringFromName('brandShortName');
		return Services.strings.createBundle(LOCALE).formatStringFromName(string, [vstring], 1);
	},
	get Brand () { // alert
		delete this.Brand;
		return this.Brand = Services.strings.createBundle('chrome://branding/locale/brand.properties');
	},
	addonDisabled: function (id, reason) { // alert
		if (reason != 1)
			as.showAlertNotification('chrome://branding/content/icon64.png', this.getString('showNotification1', NAME), this.getString('showNotification2', NAME));
		AddonManager.getAddonByID(id, function (addon) {
			addon.userDisabled = true;
		});
	}
};

function log (string) {
	cs.logStringMessage(string);
}

const rmh = { // reference manual help
	icon: 'chrome://referencemanualhelp/content/icon.png', // manifest
	mainmenu: 'Misc', // menuitem will appear here 
	menucmnd: 'rmhmenucmnd', // uniqueID
	btncmnd: 'rmhbtncmnd', // uniqueID
	get menudesc () { // menu description
		return util.getString('menu'); // properties
	},
	get statusbarmsg () { // statusbar
		return util.getString('statusbar'); // properties
	},
	showmenuname: 'rmhmenu', // menu attached to a button, left click on button
	showMenu: function () { // command function
		JSB.ShowMenu(this.showmenuname, true); // ShowMenu(string menu name, bool send command)
	},
	btnmenuname: 'rmhrmenu', // menu attached to a button, right click on button
	get btnrmenu () { // menu attached to a button, right click on button
		return (this.hasOwnProperty('btnmenuname') ? this.btnmenuname : '');
	},
	get btntooltip () { // button tooltip
		return (this.hasOwnProperty('btnmenuname') ? NAME + '\n' + util.getString('tooltip') : NAME);
	},
	observe: function (subject, topic, data) { // init
		switch (topic) {
		case util.topic:
			Services.obs.removeObserver(this, topic);
			this.load();
			break;
		case 'nsPref:changed': // AddButton
			if (data == BRANCH + 'addbutton')
				util.getPref(BRANCH + 'addbutton') ? 
				JSB.AddButton(util.getPref(BRANCH + 'toolbar'), this.btncmnd, this.btnrmenu, this.btntooltip) : 
				JSB.RemoveButton(util.getPref(BRANCH + 'toolbar'), this.btncmnd);
			break;
		case 'addon-options-displayed': // toolbar
			if (data == BRANCH.split('.')[1] + '@adodupan') {
				Services.obs.addObserver(this, 'addon-options-hidden', false); // toolbar
				Services.prefs.addObserver(BRANCH, this, false); // AddButton
				var tbprefbase = 'kmeleon.toolband.', toolbar = [];
				Services.prefs.getBranch(tbprefbase).getChildList('').forEach(function(name) {
					var tbname = name.split('.')[0], 
					tbex = ['Bookmarks', 'Favorites', 'Hotlist', 'Menu', 'Tabs', 'Throbber', 'URL Bar', '&Save Page As'];
					if (tbex.indexOf(tbname) == -1 && util.getPref(tbprefbase + tbname + '.visibility') && toolbar.indexOf(tbname) == -1)
						toolbar.push(tbname);
				});
				toolbar.sort(function(a, b) {
					return a - b;
				});
				for (var a in toolbar)	/**	menulist.appendItem(label, value, description) **/
					subject.getElementById(BRANCH.split('.')[1] + 'toolbar').appendItem(toolbar[a], toolbar[a], null);
				subject.getElementById(BRANCH.split('.')[1] + 'toolbar').selectedIndex = toolbar.indexOf(util.getPref(BRANCH + 'toolbar'));
				subject.getElementById(BRANCH.split('.')[1] + 'toolbar').disabled = util.getPref(BRANCH + 'addbutton');
			}
			break;
		case 'addon-options-hidden': // toolbar
			if (data == BRANCH.split('.')[1] + '@adodupan') {
				Services.obs.removeObserver(this, topic);
				Services.prefs.removeObserver(BRANCH, this); // AddButton
			}
			break;
		}
	},
	load: function () {
		util.installed = true;
		util.loadDefaultPrefs(util.addonPath, 'preferences.js');
		Services.obs.addObserver(this, 'addon-options-displayed', false); // toolbar
		JSB.RegisterCmd( // menu command 
			this.menucmnd, // uniqueID
			this.statusbarmsg, // statusbar
			function () { // command function
				Services.prompt.alert(null, NAME, util.getString('msg1') + '\n\n' + util.getString('msg2'));
			},
			this.icon // icon 
		);
		JSB.RegisterCmd( // button command 
			this.btncmnd, // uniqueID
			this.statusbarmsg, // statusbar
			this.showMenu.bind(this), // command function
			this.icon // icon 
		);
		JSB.SetMenuCallback(this.btnrmenu, util.getString('msgtest') + util.getString('msg3'), function () {
			Services.prompt.alert(null, NAME, util.getString('msg1') + '\n\n' + util.getString('msg3'));
		}, '');
		JSB.SetMenuCallback(this.showmenuname, util.getString('msgtest') + util.getString('msg4'), function () {
			Services.prompt.alert(null, NAME, util.getString('msg1') + '\n\n' + util.getString('msg4'));
		}, '');

		JSB.SetMenu(this.mainmenu, 1, this.menudesc, this.menucmnd, 1);
//		JSB.SetAccel('CTRL ALT SHIFT X', this.menucmnd); // key combination, if you need this, just use it
		if (Services.appinfo.appBuildID < 7501)
			JSB.SetMenu(this.mainmenu, 4, 'bmpmenu()', 1);
		if (util.getPref(BRANCH + 'addbutton'))
			JSB.AddButton(
				util.getPref(BRANCH + 'toolbar'), // toolbar name
				this.btncmnd, // command uniqueID
				this.btnrmenu, // menu attached to a button, right click on button
				this.btntooltip // tooltip
			);
	},
	unload: function () {
		if (util.installed) {
			JSB.SetMenu(this.showmenuname, 1, util.getString('msgtest') + util.getString('msg4'), 0);
			JSB.SetMenu(this.btnrmenu, 1, util.getString('msgtest') + util.getString('msg3'), 0);
			JSB.SetMenu(this.mainmenu, 1, '', this.menucmnd);
//			JSB.SetAccel('CTRL ALT SHIFT X', '');
			if (Services.appinfo.appBuildID < 7501)
				JSB.SetMenu(this.mainmenu, 4, 'bmpmenu()', 1);
			if (util.getPref(BRANCH + 'addbutton'))
				JSB.RemoveButton(util.getPref(BRANCH + 'toolbar'), this.btncmnd);
			JSB.UnregisterCmd(this.btncmnd);
			JSB.UnregisterCmd(this.menucmnd);
			Services.obs.removeObserver(this, 'addon-options-displayed'); // toolbar
			Services.strings.flushBundles();
		}
	},
	QueryInterface: XPCOMUtils.generateQI([Components.interfaces.nsIObserver])
};

const NAME = 'Reference Manual Help'; // alert
const BRANCH = 'extensions.referencemanualhelp.';
const LOCALE = 'chrome://referencemanualhelp/locale/referencemanualhelp.properties';

function startup (data, reason) {
	if (!util.jsbload(reason))
		util.addonDisabled(data.id, reason);
	else {
		util.addonPath = data.installPath;
		if (reason == 1)
			Services.obs.addObserver(rmh, util.topic, false);
		else
			rmh.load();
	}
}
function shutdown (data, reason) {
	rmh.unload();
}
function install (data, reason) {}
function uninstall (data, reason) { // after closing the about:addons tab
	if (reason == 6)
		Services.prefs.deleteBranch(BRANCH);
}